import { toTypedSchema } from "@vee-validate/zod";
import { storeToRefs } from "pinia";
import { useForm } from "vee-validate";
import { ref, watch } from "vue";
import { useRoute, useRouter } from "vue-router";

import { invertDate } from "src/functions/invertDate";
import { quasarNotify } from "src/functions/quasarNotify";
import { useCartStore } from "src/stores/cart";
import { useFormStore } from "src/stores/forms";
import { useModalStore } from "src/stores/modalStore";
import { useRangePriceStore } from "src/stores/rangePrice";
import { availabilitySchema } from "src/utils/zod/schemas";
import { rangePriceSchema } from "src/utils/zod/schemas/rangePriceSchema";

/**
 * Custom hook for managing the postal and calendar functionality.
 *
 * @param {Object} options - The options for the hook.
 * @param {string} options.modalItem - The modal item isOpenAvailability || isOpenFilters.
 * @param {string} options.type - The type of the hook. home || product || availability || filter
 * @returns {Object} - The hook functions and data.
 */
export function usePostalCalendar({ modalItem = "", type = "home" }) {
  const route = useRoute();
  const { push } = useRouter()

  const rangePriceStore = useRangePriceStore();
  const { rangeValue } = storeToRefs(rangePriceStore);

  const modalStore = useModalStore();
  const { openModal } = modalStore

  const formStore = useFormStore();
  const { sortProductFilters } = storeToRefs(formStore);

  const cartStore = useCartStore();
  const { addToCart, getProducts } = cartStore;
  const { products, homeSection } = storeToRefs(cartStore);

  const min = 0;
  const max = 200;
  const category = ref(route.path.split("/")[2])

  const { handleSubmit, handleReset, defineField, errors, setValues } = useForm(
    {
      validationSchema: toTypedSchema(
        type !== "filter" ? availabilitySchema : rangePriceSchema
      ),
      initialValues: {
        range: {
          min,
          max,
        },
        postalCode: "",
        date: "",
      },
    }
  );
  const [calendar, calendarAttrs] = defineField("date");
  const [postalCode, postalCodeAttrs] = defineField("postalCode");
  const [priceRange, priceRangeAttrs] = defineField("range");
  const [dedication, dedicationAttrs] = defineField("dedication");

  watch(errors, (newErrors) => {
    const errorsObj = {
      postalCode: () =>
        quasarNotify({ message: newErrors.postalCode, type: "erro" }),
      date: () => quasarNotify({ message: newErrors.date, type: "erro" }),
      range: () => quasarNotify({ message: newErrors.range, type: "erro" }),
      dedication: () =>
        quasarNotify({ message: newErrors.dedication, type: "erro" }),
    };
    const keys = Object.keys(newErrors);
    keys.forEach((key) => {
      errorsObj[key]();
    });
  });

  watch(
    [() => route.path, () => sortProductFilters.value],
    ([newPath]) => {
      const categoryPath = newPath.split("/")[2];
      category.value = categoryPath;
    }
  );

  const onSubmit = handleSubmit((values) => {
    const postalAndDateParams = {
      postalCode: values.postalCode,
      dateExpired: invertDate(values.date),
    };

    const categoryObj = {
      plantas: "Floranet Plantas",
      ramos: "Floranet Ramos",
    };

    const objVal = {
      home: async () => {
        console.log(type);

        await getProducts(
          {
            postalCode: values.postalCode,
            dateExpired: invertDate(values.date),
          },
          () => homeSection.value.scrollIntoView()
        );
      },
      product: async () => {
        console.log(type);

        await getProducts(postalAndDateParams);

        const hasProduct = products.value.data.some((item) => {
          const date = new Date(item.dateExpired);
          const day = date.getDate();
          const month = (date.getMonth() + 1).toString().padStart(2, '0');
          const year = date.getFullYear();
          const dateExpired = `${day}/${month}/${year}`;

          const id = +route.path.split('/')[2];

          return item.postalCode === values.postalCode && item.id === id && values.date <= dateExpired
        });

        if (!hasProduct) {
          push('/categoria/ramos')
          quasarNotify({ message: 'Seleccione una nueva fecha y un nuevo código postal.', type: 'warning' })

          setTimeout(() => {
            openModal({ modal: 'availability' })
          }, 2000)

          return
        }

        addToCart(products.value.current, dedication)
      },
      availability: async () => {
        console.log(type);

        await getProducts({
          postalCode: values.postalCode,
          dateExpired: invertDate(values.date),
        });
      },
      filter: async () => {
        console.log(type);

        rangeValue.value.max = values.range.max;
        rangeValue.value.min = values.range.min;

        const params = {
          type: categoryObj[category.value],
          minPrice: values.range.min,
          maxPrice: values.range.max,
        };

        await getProducts(params);
      },
      default: () => {
        console.error(
          `INVALID TYPE! TYPE: ${type}, ONLY HOME, PRODUCT AND FILTER ARE VALID!`
        );
      },
    };
    objVal[type]() || objVal["default"]();

    if (modalItem) {
      modalStore[modalItem] = false;
    }
    handleReset();
  });

  return {
    onSubmit,
    setValues,
    modalStore,
    fields: {
      calendar,
      calendarAttrs,
      postalCode,
      postalCodeAttrs,
      priceRange,
      priceRangeAttrs,
      dedication,
      dedicationAttrs,
    },
    errors,
  };
}
