import { defineStore } from "pinia";
import { ref } from "vue";
import { useRouter } from "vue-router";

import { apiBack } from "src/boot/axios";
import { quasarNotify } from "src/functions/quasarNotify";
import { useLocalStorage } from "src/hooks/useLocalStorage";

export const useCartStore = defineStore("cart", () => {
  const { push } = useRouter();
  const { addItem, getItem, removeItem } = useLocalStorage()

  //! Elements
  const checkoutRef = ref(null);
  const homeSection = ref(null);

  const initialValues = [{
    id: null,
    name: "",
    price: null,
    image: "",
    description: "",
    dateExpired: "",
    isNew: null,
    type: "",
    postalCode: "",
    order_position: null,
    recommend: null
  }]

  //! Variables
  const cart = ref([]);

  (() => {
    cart.value = getItem('cart');
  })()

  const addCartLoadingBtn = ref(false);
  const routeId = ref(null);
  const products = ref({
    data: initialValues,
    prev: initialValues,
    current: initialValues,
    next: initialValues,
  });
  const featuredProducts = ref({
    page: undefined,
    productsPerPage: undefined,
    products: [],
  });

  /**
   * Transforms options object into params object.
   *
   * @param {Object} options - The options object.
   * @param {number} options.itens - The items array.
   * @param {boolean} options.featured - The featured flag.
   * @param {number} options.page - The page number.
   * @param {string} options.type - The type name.
   * @param {string} options.postalCode - The postal code.
   * @param {string} options.dateExpired - The expiration date.
   * @param {number} options.minPrice - The minimum price.
   * @param {number} options.maxPrice - The maximum price.
   * @param {number} options.bigPrice - The big price.
   * @param {number} options.lowPrice - The low price.
   * @param {boolean} options.isNew - The new flag.
   * @returns {Object} - The params object.
   */
  function transformOptionsToParams(
    options = {
      postalCode: undefined,
      dateExpired: undefined,
      type: undefined,
      minPrice: undefined,
      maxPrice: undefined,
      bigPrice: undefined,
      lowPrice: undefined,
      isNew: undefined,
      order_crescent: undefined,
      order_descending: undefined,
      recommend: undefined,
    }
  ) {
    const optionsObj = {
      postalCode: options.postalCode,
      dateExpired: options.dateExpired,
      type: options.type,
      minPrice: options.minPrice,
      maxPrice: options.maxPrice,
      bigPrice: options.bigPrice,
      lowPrice: options.lowPrice,
      isNew: options.isNew,
      order_crescent: options.order_crescent,
      order_descending: options.order_descending,
      recommend: options.recommend,
    };
    const validKeys = Object.keys(options).filter(
      (key) => options[key] !== undefined
    );
    const params = validKeys.reduce((acc, key) => {
      acc[key] = optionsObj[key];
      return acc;
    }, {});

    return params;
  }

  /**
   * Fetches products based on the provided options.
   *
   * @param {Object} options - The options for fetching products.
   * @param {number} options.itens - The items to fetch.
   * @param {boolean} options.featured - Whether to fetch only featured products.
   * @param {number} options.page - The page number to fetch.
   * @param {string} options.type - The type of products to fetch.
   * @param {string} options.postalCode - The postal code for filtering products.
   * @param {string} options.dateExpired - The expiration date for filtering products.
   * @param {number} options.minPrice - The minimum price for filtering products.
   * @param {number} options.maxPrice - The maximum price for filtering products.
   * @param {number} options.bigPrice - The big price for filtering products.
   * @param {number} options.lowPrice - The low price for filtering products.
   * @param {boolean} options.isNew - Whether to fetch only new products.
   * @param {Function} navigate - The navigation function to call after fetching products.
   * @returns {Promise<void>} - A promise that resolves when the products are fetched.
   */
  async function getProducts(
    options = {
      postalCode: undefined,
      dateExpired: undefined,
      type: undefined,
      minPrice: undefined,
      maxPrice: undefined,
      bigPrice: undefined,
      lowPrice: undefined,
      isNew: undefined,
      order_crescent: undefined,
      order_descending: undefined,
      recommend: undefined,
    },
    scrollIntoView = () => { }
  ) {
    const params = transformOptionsToParams(options);
    console.log(params);

    try {
      const { data: { data } } = await apiBack.get("products", { params });


      if (data.length === 0) {
        return quasarNotify({
          message:
            "No hay productos disponibles para la fecha y el código postal seleccionados",
          type: "erro",
        });
      }

      products.value.data = data;

      if (scrollIntoView) {
        scrollIntoView();
      }

      console.groupCollapsed("%c PRODUCTS FETCHED!", "color: green;");
      console.groupCollapsed("%c PRODUCTS DATA", "color: tomato;");
      console.table(products.value.data);
      console.groupEnd();
      console.groupCollapsed("%c PREV PRODUCT", "color: tomato;");
      console.table(products.value.prev);
      console.groupEnd();
      console.groupCollapsed(
        `%c CURRENT PRODUCT: ${products.value.current.slug}`,
        "color: tomato;"
      );
      console.table(products.value.current);
      console.groupEnd();
      console.groupCollapsed("%c NEXT PRODUCT", "color: tomato;");
      console.table(products.value.next);
      console.groupEnd();
      console.groupEnd();
    } catch (err) {
      new Error(`FATAL ERROR ::: ${err}`);
    }
  }

  /**
   * Fetches a product by its ID and updates the cart state.
   *
   * @param {string} id - The ID of the product to fetch.
   * @param {object} options - Additional options for the product fetch.
   * @param {string} options.type - The type of the product.
   * @param {string} options.postalCode - The postal code for location-based filtering.
   * @param {string} options.dateExpired - The expiration date for time-based filtering.
   * @param {boolean} debug - Flag indicating whether to enable debug mode.
   * @returns {Promise<void>} - A promise that resolves when the product is fetched and the cart state is updated.
   */
  async function getProduct(
    id,
    options = {
      type: undefined,
      postalCode: undefined,
      dateExpired: undefined,
    },
    debug = false
  ) {
    if (id) {
      routeId.value = id;
      try {
        const params = transformOptionsToParams(options);

        const promises = [
          apiBack.get(`products/${+id - 1}`),
          apiBack.get(`products/${+id}`),
          apiBack.get(`products/${+id + 1}`),
        ];
        const results = await Promise.allSettled(promises);
        const [prev, current, next] = results.map((res) => {
          const result = {
            fulfilled: res.value?.data,
            rejected: res.reason,
          };

          return result[res.status].data[0];
        })

        products.value.prev = prev;
        products.value.current = current;
        products.value.next = next;

        if (!current) {
          push({ name: "NotFound" })
        }

        if (debug) {
          console.groupCollapsed(
            `%c PRODUCT FETCHED! SLUG: ${routeId.value}`,
            "color: green;"
          );
          console.table(products.value.prev);
          console.table(products.value.current);
          console.table(products.value.next);
          console.groupEnd();
        }

        if (products.value.current.response?.status === 404) {
          push({ name: "NotFound" });
        }
      } catch (err) {
        console.error(`FATAL ERROR ::: ${err}`);
      }
    }
  }

  /**
   * Retrieves featured products based on the provided options.
   *
   * @param {Object} options - The options for retrieving featured products.
   * @param {number} options.itens - The number of items to retrieve.
   * @param {number} options.featured - The flag indicating if the products should be featured.
   * @param {number} [options.page] - The page number for pagination.
   * @param {string} [options.type] - The type of the products.
   * @param {string} [options.postalCode] - The postal code for location-based filtering.
   * @param {string} [options.dateExpired] - The expiration date for filtering.
   * @param {number} [options.minPrice] - The minimum price for filtering.
   * @param {number} [options.maxPrice] - The maximum price for filtering.
   * @param {number} [options.bigPrice] - The big price for filtering.
   * @param {number} [options.lowPrice] - The low price for filtering.
   * @param {boolean} [options.isNew] - The flag indicating if the products are new.
   * @returns {Promise<void>} - A promise that resolves when the featured products are retrieved.
   */
  async function getFeaturedProducts(
    options = {
      postalCode: undefined,
      dateExpired: undefined,
      type: undefined,
      minPrice: undefined,
      maxPrice: undefined,
      bigPrice: undefined,
      lowPrice: undefined,
      isNew: undefined,
      order_crescent: undefined,
      order_descending: undefined,
      recommend: 1,
    },
    debug = false
  ) {
    try {
      const params = transformOptionsToParams(options);

      (async () => {
        const {
          data: { data },
        } = await apiBack.get("products", { params });
        featuredProducts.value = data[0];

        if (debug) {
          console.groupCollapsed(
            "%c FEATURED PRODUCTS FETCHED!",
            "color: green;"
          );
          console.table(data.products);
          console.groupEnd();
        }
      })();
    } catch (err) {
      new Error(`FATAL ERROR ::: ${err}`);
    }
  }

  /**
   * Adiciona um produto ao carrinho.
   * @param {Object} product - O produto a ser adicionado.
   * @param {string} dedication - A dedicação associada ao produto.
   */
  function addToCart(product, dedication) {
    const existingProduct = cart.value.find(p => p.id === product.id);
    console.log(existingProduct)

    if (!existingProduct) {
      const arr = [...cart.value];
      arr.push(product);
      console.log(arr)
      addItem("cart", JSON.stringify(arr));
      quasarNotify({ message: 'Producto añadido al carrito.', type: 'success' })
      return
    }
    quasarNotify({
      message: "Este producto ya está en el carrito",
      type: "info",
    });
  }

  /**
   * Remove an item from the cart by its ID.
   * @param {number} id - The ID of the item to be removed.
   */
  function removeFromCart(id) {
    const newArrRemovedItem = cart.value.filter((p) => id !== p.id);
    addItem("cart", JSON.stringify(newArrRemovedItem))
  }

  return {
    checkoutRef,
    homeSection,

    cart,
    addCartLoadingBtn,
    products,
    featuredProducts,

    getFeaturedProducts,
    getProducts,
    addToCart,
    removeFromCart,
    getProduct,
  };
});
